/*
 * Author: Kipp Cannon <kipp@gravity.phys.uwm.edu>
 * Revision: $Id: FrequencySeries.m4,v 1.3 2005/06/04 18:15:06 kipp Exp $
 */

#include <string.h>
#include <lal/Date.h>
#include <lal/FrequencySeries.h>
#include <lal/LALDatatypes.h>
#include <lal/LALStdlib.h>
#include <lal/LALStatusMacros.h>
#include <lal/LALErrno.h>
#include <lal/LALError.h>
#include <lal/Sequence.h>
#include <lal/Units.h>
#include <lal/XLALError.h>

NRCSID(FREQUENCYSERIESC, "$Id: FrequencySeries.m4,v 1.3 2005/06/04 18:15:06 kipp Exp $");




void XLALDestroyREAL4FrequencySeries (
	REAL4FrequencySeries *series
)
{
	if(series)
		XLALDestroyREAL4Sequence (series->data);
	LALFree(series);
}


void LALDestroyREAL4FrequencySeries (
	LALStatus *status,
	REAL4FrequencySeries *series
)
{
	INITSTATUS(status, "LALDestroyREAL4FrequencySeries", FREQUENCYSERIESC);
	XLALDestroyREAL4FrequencySeries (series);
	RETURN(status);
}


REAL4FrequencySeries *XLALCreateREAL4FrequencySeries (
	const CHAR *name,
	const LIGOTimeGPS *epoch,
	REAL8 f0,
	REAL8 deltaF,
	const LALUnit *sampleUnits,
	size_t length
)
{
	static const char *func = "XLALCreateREAL4FrequencySeries";
	REAL4FrequencySeries *new;
	REAL4Sequence *sequence;

	new = LALMalloc(sizeof(*new));
	sequence = XLALCreateREAL4Sequence (length);
	if(!new || !sequence) {
		LALFree(new);
		XLALDestroyREAL4Sequence (sequence);
		XLAL_ERROR_NULL(func, XLAL_ENOMEM);
	}

	if(name)
		strncpy(new->name, name, LALNameLength);
	else
		new->name[0] = '\0';
	new->epoch = *epoch;
	new->f0 = f0;
	new->deltaF = deltaF;
	new->sampleUnits = *sampleUnits;
	new->data = sequence;

	return(new);
}


void LALCreateREAL4FrequencySeries (
	LALStatus *status,
	REAL4FrequencySeries **output,
	const CHAR *name,
	LIGOTimeGPS epoch,
	REAL8 f0,
	REAL8 deltaF,
	LALUnit sampleUnits,
	size_t length
)
{
	INITSTATUS(status, "LALCreateREAL4FrequencySeries", FREQUENCYSERIESC);
	ASSERT(output != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	*output = XLALCreateREAL4FrequencySeries (name, &epoch, f0, deltaF, &sampleUnits, length);
	if(*output == NULL) {
		XLALClearErrno();
		ABORT(status, LAL_NOMEM_ERR, LAL_NOMEM_MSG);
	}
	RETURN(status);
}


REAL4FrequencySeries *XLALCutREAL4FrequencySeries (
	const REAL4FrequencySeries *series,
	size_t first,
	size_t length
)
{
	static const char *func = "XLALCutREAL4FrequencySeries";
	REAL4FrequencySeries *new;
	REAL4Sequence *sequence;

	if(!series || !series->data)
		return(NULL);

	new = LALMalloc(sizeof(*new));
	sequence = XLALCutREAL4Sequence (series->data, first, length);
	if(!new || !sequence) {
		LALFree(new);
		XLALDestroyREAL4Sequence (sequence);
		XLAL_ERROR_NULL(func, XLAL_ENOMEM);
	}

	*new = *series;
	new->data = sequence;
	new->f0 += first * new->deltaF;

	return(new);
}


size_t XLALResizeREAL4FrequencySeries (
	REAL4FrequencySeries *series,
	int first,
	size_t length
)
{
	if(!series)
		return(0);

	series->f0 += first * series->deltaF;
	return(XLALResizeREAL4Sequence (series->data, first, length));
}


size_t XLALShrinkREAL4FrequencySeries (
	REAL4FrequencySeries *series,
	size_t first,
	size_t length
)
{
	return(XLALResizeREAL4FrequencySeries (series, first, length));
}


void LALShrinkREAL4FrequencySeries (
	LALStatus *status,
	REAL4FrequencySeries *series,
	size_t first,
	size_t length
)
{
	size_t new_length;

	INITSTATUS(status, "LALShrinkREAL4FrequencySeries", FREQUENCYSERIESC);

	ASSERT(series != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(series->data != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(first + length <= series->data->length, status, LAL_RANGE_ERR, LAL_RANGE_MSG);
	new_length = XLALShrinkREAL4FrequencySeries (series, first, length);
	if(new_length != length) {
		XLALClearErrno();
		ABORT(status, LAL_FAIL_ERR, LAL_FAIL_MSG);
	}
	RETURN(status);
}


REAL4FrequencySeries *XLALAddREAL4FrequencySeries (
	REAL4FrequencySeries *arg1,
	const REAL4FrequencySeries *arg2
)
{
	static const char *func = "XLALAddREAL4FrequencySeries";
	int offset = (arg2->f0 - arg1->f0) / arg1->deltaF;
	REAL8 ratio = XLALUnitRatio(&arg1->sampleUnits, &arg2->sampleUnits);
	unsigned int i;

	/* make sure arguments are compatible */
	if(XLALGPSCmp(&arg1->epoch, &arg2->epoch) || (arg1->deltaF != arg2->deltaF) || XLALIsREAL8FailNaN(ratio))
		XLAL_ERROR_NULL(func, XLAL_EDATA);
	/* FIXME: generalize to relax this requirement */
	if((arg2->f0 < arg1->f0) || (offset + arg2->data->length > arg1->data->length))
		XLAL_ERROR_NULL(func, XLAL_EBADLEN);
	
	/* add arg2 to arg1, adjusting the units */
	for(i = 0; i < arg2->data->length; i++) {
		arg1->data->data[offset + i] += arg2->data->data[i] / ratio;
	}

	return(arg1);
}


REAL4FrequencySeries *XLALSubtractREAL4FrequencySeries (
	REAL4FrequencySeries *arg1,
	const REAL4FrequencySeries *arg2
)
{
	static const char *func = "XLALAddREAL4FrequencySeries";
	int offset = (arg2->f0 - arg1->f0) / arg1->deltaF;
	REAL8 ratio = XLALUnitRatio(&arg1->sampleUnits, &arg2->sampleUnits);
	unsigned int i;

	/* make sure arguments are compatible */
	if(XLALGPSCmp(&arg1->epoch, &arg2->epoch) || (arg1->deltaF != arg2->deltaF) || XLALIsREAL8FailNaN(ratio))
		XLAL_ERROR_NULL(func, XLAL_EDATA);
	/* FIXME: generalize to relax this requirement */
	if((arg2->f0 < arg1->f0) || (offset + arg2->data->length > arg1->data->length))
		XLAL_ERROR_NULL(func, XLAL_EBADLEN);
	
	/* subtract arg2 from arg1, adjusting the units */
	for(i = 0; i < arg2->data->length; i++) {
		arg1->data->data[offset + i] -= arg2->data->data[i] / ratio;
	}

	return(arg1);
}





REAL4FrequencySeries *XLALMultiplyREAL4FrequencySeries (
	REAL4FrequencySeries *arg1,
	const REAL4FrequencySeries *arg2
)
{
	static const char *func = "XLALMultiplyREAL4FrequencySeries";
	int offset = (arg2->f0 - arg1->f0) / arg1->deltaF;
	REAL8 ratio = XLALUnitRatio(&arg1->sampleUnits, &arg2->sampleUnits);
	unsigned int i;

	/* make sure arguments are compatible */
	if(XLALGPSCmp(&arg1->epoch, &arg2->epoch) || (arg1->deltaF != arg2->deltaF) || XLALIsREAL8FailNaN(ratio))
		XLAL_ERROR_NULL(func, XLAL_EDATA);
	/* FIXME: generalize to relax this requirement */
	if((arg2->f0 < arg1->f0) || (offset + arg2->data->length > arg1->data->length))
		XLAL_ERROR_NULL(func, XLAL_EBADLEN);
	
	/* multiply arg2 by arg1, adjusting the units */
	for(i = 0; i < arg2->data->length; i++) {
		arg1->data->data[offset + i] *= arg2->data->data[i] / ratio;
	}

	return(arg1);
}






void XLALDestroyREAL8FrequencySeries (
	REAL8FrequencySeries *series
)
{
	if(series)
		XLALDestroyREAL8Sequence (series->data);
	LALFree(series);
}


void LALDestroyREAL8FrequencySeries (
	LALStatus *status,
	REAL8FrequencySeries *series
)
{
	INITSTATUS(status, "LALDestroyREAL8FrequencySeries", FREQUENCYSERIESC);
	XLALDestroyREAL8FrequencySeries (series);
	RETURN(status);
}


REAL8FrequencySeries *XLALCreateREAL8FrequencySeries (
	const CHAR *name,
	const LIGOTimeGPS *epoch,
	REAL8 f0,
	REAL8 deltaF,
	const LALUnit *sampleUnits,
	size_t length
)
{
	static const char *func = "XLALCreateREAL8FrequencySeries";
	REAL8FrequencySeries *new;
	REAL8Sequence *sequence;

	new = LALMalloc(sizeof(*new));
	sequence = XLALCreateREAL8Sequence (length);
	if(!new || !sequence) {
		LALFree(new);
		XLALDestroyREAL8Sequence (sequence);
		XLAL_ERROR_NULL(func, XLAL_ENOMEM);
	}

	if(name)
		strncpy(new->name, name, LALNameLength);
	else
		new->name[0] = '\0';
	new->epoch = *epoch;
	new->f0 = f0;
	new->deltaF = deltaF;
	new->sampleUnits = *sampleUnits;
	new->data = sequence;

	return(new);
}


void LALCreateREAL8FrequencySeries (
	LALStatus *status,
	REAL8FrequencySeries **output,
	const CHAR *name,
	LIGOTimeGPS epoch,
	REAL8 f0,
	REAL8 deltaF,
	LALUnit sampleUnits,
	size_t length
)
{
	INITSTATUS(status, "LALCreateREAL8FrequencySeries", FREQUENCYSERIESC);
	ASSERT(output != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	*output = XLALCreateREAL8FrequencySeries (name, &epoch, f0, deltaF, &sampleUnits, length);
	if(*output == NULL) {
		XLALClearErrno();
		ABORT(status, LAL_NOMEM_ERR, LAL_NOMEM_MSG);
	}
	RETURN(status);
}


REAL8FrequencySeries *XLALCutREAL8FrequencySeries (
	const REAL8FrequencySeries *series,
	size_t first,
	size_t length
)
{
	static const char *func = "XLALCutREAL8FrequencySeries";
	REAL8FrequencySeries *new;
	REAL8Sequence *sequence;

	if(!series || !series->data)
		return(NULL);

	new = LALMalloc(sizeof(*new));
	sequence = XLALCutREAL8Sequence (series->data, first, length);
	if(!new || !sequence) {
		LALFree(new);
		XLALDestroyREAL8Sequence (sequence);
		XLAL_ERROR_NULL(func, XLAL_ENOMEM);
	}

	*new = *series;
	new->data = sequence;
	new->f0 += first * new->deltaF;

	return(new);
}


size_t XLALResizeREAL8FrequencySeries (
	REAL8FrequencySeries *series,
	int first,
	size_t length
)
{
	if(!series)
		return(0);

	series->f0 += first * series->deltaF;
	return(XLALResizeREAL8Sequence (series->data, first, length));
}


size_t XLALShrinkREAL8FrequencySeries (
	REAL8FrequencySeries *series,
	size_t first,
	size_t length
)
{
	return(XLALResizeREAL8FrequencySeries (series, first, length));
}


void LALShrinkREAL8FrequencySeries (
	LALStatus *status,
	REAL8FrequencySeries *series,
	size_t first,
	size_t length
)
{
	size_t new_length;

	INITSTATUS(status, "LALShrinkREAL8FrequencySeries", FREQUENCYSERIESC);

	ASSERT(series != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(series->data != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(first + length <= series->data->length, status, LAL_RANGE_ERR, LAL_RANGE_MSG);
	new_length = XLALShrinkREAL8FrequencySeries (series, first, length);
	if(new_length != length) {
		XLALClearErrno();
		ABORT(status, LAL_FAIL_ERR, LAL_FAIL_MSG);
	}
	RETURN(status);
}


REAL8FrequencySeries *XLALAddREAL8FrequencySeries (
	REAL8FrequencySeries *arg1,
	const REAL8FrequencySeries *arg2
)
{
	static const char *func = "XLALAddREAL8FrequencySeries";
	int offset = (arg2->f0 - arg1->f0) / arg1->deltaF;
	REAL8 ratio = XLALUnitRatio(&arg1->sampleUnits, &arg2->sampleUnits);
	unsigned int i;

	/* make sure arguments are compatible */
	if(XLALGPSCmp(&arg1->epoch, &arg2->epoch) || (arg1->deltaF != arg2->deltaF) || XLALIsREAL8FailNaN(ratio))
		XLAL_ERROR_NULL(func, XLAL_EDATA);
	/* FIXME: generalize to relax this requirement */
	if((arg2->f0 < arg1->f0) || (offset + arg2->data->length > arg1->data->length))
		XLAL_ERROR_NULL(func, XLAL_EBADLEN);
	
	/* add arg2 to arg1, adjusting the units */
	for(i = 0; i < arg2->data->length; i++) {
		arg1->data->data[offset + i] += arg2->data->data[i] / ratio;
	}

	return(arg1);
}


REAL8FrequencySeries *XLALSubtractREAL8FrequencySeries (
	REAL8FrequencySeries *arg1,
	const REAL8FrequencySeries *arg2
)
{
	static const char *func = "XLALAddREAL8FrequencySeries";
	int offset = (arg2->f0 - arg1->f0) / arg1->deltaF;
	REAL8 ratio = XLALUnitRatio(&arg1->sampleUnits, &arg2->sampleUnits);
	unsigned int i;

	/* make sure arguments are compatible */
	if(XLALGPSCmp(&arg1->epoch, &arg2->epoch) || (arg1->deltaF != arg2->deltaF) || XLALIsREAL8FailNaN(ratio))
		XLAL_ERROR_NULL(func, XLAL_EDATA);
	/* FIXME: generalize to relax this requirement */
	if((arg2->f0 < arg1->f0) || (offset + arg2->data->length > arg1->data->length))
		XLAL_ERROR_NULL(func, XLAL_EBADLEN);
	
	/* subtract arg2 from arg1, adjusting the units */
	for(i = 0; i < arg2->data->length; i++) {
		arg1->data->data[offset + i] -= arg2->data->data[i] / ratio;
	}

	return(arg1);
}





REAL8FrequencySeries *XLALMultiplyREAL8FrequencySeries (
	REAL8FrequencySeries *arg1,
	const REAL8FrequencySeries *arg2
)
{
	static const char *func = "XLALMultiplyREAL8FrequencySeries";
	int offset = (arg2->f0 - arg1->f0) / arg1->deltaF;
	REAL8 ratio = XLALUnitRatio(&arg1->sampleUnits, &arg2->sampleUnits);
	unsigned int i;

	/* make sure arguments are compatible */
	if(XLALGPSCmp(&arg1->epoch, &arg2->epoch) || (arg1->deltaF != arg2->deltaF) || XLALIsREAL8FailNaN(ratio))
		XLAL_ERROR_NULL(func, XLAL_EDATA);
	/* FIXME: generalize to relax this requirement */
	if((arg2->f0 < arg1->f0) || (offset + arg2->data->length > arg1->data->length))
		XLAL_ERROR_NULL(func, XLAL_EBADLEN);
	
	/* multiply arg2 by arg1, adjusting the units */
	for(i = 0; i < arg2->data->length; i++) {
		arg1->data->data[offset + i] *= arg2->data->data[i] / ratio;
	}

	return(arg1);
}






void XLALDestroyCOMPLEX8FrequencySeries (
	COMPLEX8FrequencySeries *series
)
{
	if(series)
		XLALDestroyCOMPLEX8Sequence (series->data);
	LALFree(series);
}


void LALDestroyCOMPLEX8FrequencySeries (
	LALStatus *status,
	COMPLEX8FrequencySeries *series
)
{
	INITSTATUS(status, "LALDestroyCOMPLEX8FrequencySeries", FREQUENCYSERIESC);
	XLALDestroyCOMPLEX8FrequencySeries (series);
	RETURN(status);
}


COMPLEX8FrequencySeries *XLALCreateCOMPLEX8FrequencySeries (
	const CHAR *name,
	const LIGOTimeGPS *epoch,
	REAL8 f0,
	REAL8 deltaF,
	const LALUnit *sampleUnits,
	size_t length
)
{
	static const char *func = "XLALCreateCOMPLEX8FrequencySeries";
	COMPLEX8FrequencySeries *new;
	COMPLEX8Sequence *sequence;

	new = LALMalloc(sizeof(*new));
	sequence = XLALCreateCOMPLEX8Sequence (length);
	if(!new || !sequence) {
		LALFree(new);
		XLALDestroyCOMPLEX8Sequence (sequence);
		XLAL_ERROR_NULL(func, XLAL_ENOMEM);
	}

	if(name)
		strncpy(new->name, name, LALNameLength);
	else
		new->name[0] = '\0';
	new->epoch = *epoch;
	new->f0 = f0;
	new->deltaF = deltaF;
	new->sampleUnits = *sampleUnits;
	new->data = sequence;

	return(new);
}


void LALCreateCOMPLEX8FrequencySeries (
	LALStatus *status,
	COMPLEX8FrequencySeries **output,
	const CHAR *name,
	LIGOTimeGPS epoch,
	REAL8 f0,
	REAL8 deltaF,
	LALUnit sampleUnits,
	size_t length
)
{
	INITSTATUS(status, "LALCreateCOMPLEX8FrequencySeries", FREQUENCYSERIESC);
	ASSERT(output != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	*output = XLALCreateCOMPLEX8FrequencySeries (name, &epoch, f0, deltaF, &sampleUnits, length);
	if(*output == NULL) {
		XLALClearErrno();
		ABORT(status, LAL_NOMEM_ERR, LAL_NOMEM_MSG);
	}
	RETURN(status);
}


COMPLEX8FrequencySeries *XLALCutCOMPLEX8FrequencySeries (
	const COMPLEX8FrequencySeries *series,
	size_t first,
	size_t length
)
{
	static const char *func = "XLALCutCOMPLEX8FrequencySeries";
	COMPLEX8FrequencySeries *new;
	COMPLEX8Sequence *sequence;

	if(!series || !series->data)
		return(NULL);

	new = LALMalloc(sizeof(*new));
	sequence = XLALCutCOMPLEX8Sequence (series->data, first, length);
	if(!new || !sequence) {
		LALFree(new);
		XLALDestroyCOMPLEX8Sequence (sequence);
		XLAL_ERROR_NULL(func, XLAL_ENOMEM);
	}

	*new = *series;
	new->data = sequence;
	new->f0 += first * new->deltaF;

	return(new);
}


size_t XLALResizeCOMPLEX8FrequencySeries (
	COMPLEX8FrequencySeries *series,
	int first,
	size_t length
)
{
	if(!series)
		return(0);

	series->f0 += first * series->deltaF;
	return(XLALResizeCOMPLEX8Sequence (series->data, first, length));
}


size_t XLALShrinkCOMPLEX8FrequencySeries (
	COMPLEX8FrequencySeries *series,
	size_t first,
	size_t length
)
{
	return(XLALResizeCOMPLEX8FrequencySeries (series, first, length));
}


void LALShrinkCOMPLEX8FrequencySeries (
	LALStatus *status,
	COMPLEX8FrequencySeries *series,
	size_t first,
	size_t length
)
{
	size_t new_length;

	INITSTATUS(status, "LALShrinkCOMPLEX8FrequencySeries", FREQUENCYSERIESC);

	ASSERT(series != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(series->data != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(first + length <= series->data->length, status, LAL_RANGE_ERR, LAL_RANGE_MSG);
	new_length = XLALShrinkCOMPLEX8FrequencySeries (series, first, length);
	if(new_length != length) {
		XLALClearErrno();
		ABORT(status, LAL_FAIL_ERR, LAL_FAIL_MSG);
	}
	RETURN(status);
}


COMPLEX8FrequencySeries *XLALAddCOMPLEX8FrequencySeries (
	COMPLEX8FrequencySeries *arg1,
	const COMPLEX8FrequencySeries *arg2
)
{
	static const char *func = "XLALAddCOMPLEX8FrequencySeries";
	int offset = (arg2->f0 - arg1->f0) / arg1->deltaF;
	REAL8 ratio = XLALUnitRatio(&arg1->sampleUnits, &arg2->sampleUnits);
	unsigned int i;

	/* make sure arguments are compatible */
	if(XLALGPSCmp(&arg1->epoch, &arg2->epoch) || (arg1->deltaF != arg2->deltaF) || XLALIsREAL8FailNaN(ratio))
		XLAL_ERROR_NULL(func, XLAL_EDATA);
	/* FIXME: generalize to relax this requirement */
	if((arg2->f0 < arg1->f0) || (offset + arg2->data->length > arg1->data->length))
		XLAL_ERROR_NULL(func, XLAL_EBADLEN);
	
	/* add arg2 to arg1, adjusting the units */
	for(i = 0; i < arg2->data->length; i++) {
		arg1->data->data[offset + i].re += arg2->data->data[i].re / ratio;
		arg1->data->data[offset + i].im += arg2->data->data[i].im / ratio;
		
	}

	return(arg1);
}


COMPLEX8FrequencySeries *XLALSubtractCOMPLEX8FrequencySeries (
	COMPLEX8FrequencySeries *arg1,
	const COMPLEX8FrequencySeries *arg2
)
{
	static const char *func = "XLALAddCOMPLEX8FrequencySeries";
	int offset = (arg2->f0 - arg1->f0) / arg1->deltaF;
	REAL8 ratio = XLALUnitRatio(&arg1->sampleUnits, &arg2->sampleUnits);
	unsigned int i;

	/* make sure arguments are compatible */
	if(XLALGPSCmp(&arg1->epoch, &arg2->epoch) || (arg1->deltaF != arg2->deltaF) || XLALIsREAL8FailNaN(ratio))
		XLAL_ERROR_NULL(func, XLAL_EDATA);
	/* FIXME: generalize to relax this requirement */
	if((arg2->f0 < arg1->f0) || (offset + arg2->data->length > arg1->data->length))
		XLAL_ERROR_NULL(func, XLAL_EBADLEN);
	
	/* subtract arg2 from arg1, adjusting the units */
	for(i = 0; i < arg2->data->length; i++) {
		arg1->data->data[offset + i].re -= arg2->data->data[i].re / ratio;
		arg1->data->data[offset + i].im -= arg2->data->data[i].im / ratio;
		
	}

	return(arg1);
}


COMPLEX8FrequencySeries *XLALConjugateCOMPLEX8FrequencySeries (
	COMPLEX8FrequencySeries *series
)
{
	XLALConjugateCOMPLEX8Sequence (series->data);
	return(series);

}



COMPLEX8FrequencySeries *XLALMultiplyCOMPLEX8FrequencySeries (
	COMPLEX8FrequencySeries *arg1,
	const COMPLEX8FrequencySeries *arg2
)
{
	static const char *func = "XLALMultiplyCOMPLEX8FrequencySeries";
	int offset = (arg2->f0 - arg1->f0) / arg1->deltaF;
	REAL8 ratio = XLALUnitRatio(&arg1->sampleUnits, &arg2->sampleUnits);
	unsigned int i;

	/* make sure arguments are compatible */
	if(XLALGPSCmp(&arg1->epoch, &arg2->epoch) || (arg1->deltaF != arg2->deltaF) || XLALIsREAL8FailNaN(ratio))
		XLAL_ERROR_NULL(func, XLAL_EDATA);
	/* FIXME: generalize to relax this requirement */
	if((arg2->f0 < arg1->f0) || (offset + arg2->data->length > arg1->data->length))
		XLAL_ERROR_NULL(func, XLAL_EBADLEN);
	
	/* multiply arg2 by arg1, adjusting the units */
	for(i = 0; i < arg2->data->length; i++) {
		REAL4 re = arg2->data->data[i].re / ratio;
		REAL4 im = arg2->data->data[i].im / ratio;
		arg1->data->data[offset + i].re = arg1->data->data[offset + i].re * re - arg1->data->data[offset + i].im * im;
		arg1->data->data[offset + i].im = arg1->data->data[offset + i].re * im + arg1->data->data[offset + i].im * re;
		
	}

	return(arg1);
}






void XLALDestroyCOMPLEX16FrequencySeries (
	COMPLEX16FrequencySeries *series
)
{
	if(series)
		XLALDestroyCOMPLEX16Sequence (series->data);
	LALFree(series);
}


void LALDestroyCOMPLEX16FrequencySeries (
	LALStatus *status,
	COMPLEX16FrequencySeries *series
)
{
	INITSTATUS(status, "LALDestroyCOMPLEX16FrequencySeries", FREQUENCYSERIESC);
	XLALDestroyCOMPLEX16FrequencySeries (series);
	RETURN(status);
}


COMPLEX16FrequencySeries *XLALCreateCOMPLEX16FrequencySeries (
	const CHAR *name,
	const LIGOTimeGPS *epoch,
	REAL8 f0,
	REAL8 deltaF,
	const LALUnit *sampleUnits,
	size_t length
)
{
	static const char *func = "XLALCreateCOMPLEX16FrequencySeries";
	COMPLEX16FrequencySeries *new;
	COMPLEX16Sequence *sequence;

	new = LALMalloc(sizeof(*new));
	sequence = XLALCreateCOMPLEX16Sequence (length);
	if(!new || !sequence) {
		LALFree(new);
		XLALDestroyCOMPLEX16Sequence (sequence);
		XLAL_ERROR_NULL(func, XLAL_ENOMEM);
	}

	if(name)
		strncpy(new->name, name, LALNameLength);
	else
		new->name[0] = '\0';
	new->epoch = *epoch;
	new->f0 = f0;
	new->deltaF = deltaF;
	new->sampleUnits = *sampleUnits;
	new->data = sequence;

	return(new);
}


void LALCreateCOMPLEX16FrequencySeries (
	LALStatus *status,
	COMPLEX16FrequencySeries **output,
	const CHAR *name,
	LIGOTimeGPS epoch,
	REAL8 f0,
	REAL8 deltaF,
	LALUnit sampleUnits,
	size_t length
)
{
	INITSTATUS(status, "LALCreateCOMPLEX16FrequencySeries", FREQUENCYSERIESC);
	ASSERT(output != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	*output = XLALCreateCOMPLEX16FrequencySeries (name, &epoch, f0, deltaF, &sampleUnits, length);
	if(*output == NULL) {
		XLALClearErrno();
		ABORT(status, LAL_NOMEM_ERR, LAL_NOMEM_MSG);
	}
	RETURN(status);
}


COMPLEX16FrequencySeries *XLALCutCOMPLEX16FrequencySeries (
	const COMPLEX16FrequencySeries *series,
	size_t first,
	size_t length
)
{
	static const char *func = "XLALCutCOMPLEX16FrequencySeries";
	COMPLEX16FrequencySeries *new;
	COMPLEX16Sequence *sequence;

	if(!series || !series->data)
		return(NULL);

	new = LALMalloc(sizeof(*new));
	sequence = XLALCutCOMPLEX16Sequence (series->data, first, length);
	if(!new || !sequence) {
		LALFree(new);
		XLALDestroyCOMPLEX16Sequence (sequence);
		XLAL_ERROR_NULL(func, XLAL_ENOMEM);
	}

	*new = *series;
	new->data = sequence;
	new->f0 += first * new->deltaF;

	return(new);
}


size_t XLALResizeCOMPLEX16FrequencySeries (
	COMPLEX16FrequencySeries *series,
	int first,
	size_t length
)
{
	if(!series)
		return(0);

	series->f0 += first * series->deltaF;
	return(XLALResizeCOMPLEX16Sequence (series->data, first, length));
}


size_t XLALShrinkCOMPLEX16FrequencySeries (
	COMPLEX16FrequencySeries *series,
	size_t first,
	size_t length
)
{
	return(XLALResizeCOMPLEX16FrequencySeries (series, first, length));
}


void LALShrinkCOMPLEX16FrequencySeries (
	LALStatus *status,
	COMPLEX16FrequencySeries *series,
	size_t first,
	size_t length
)
{
	size_t new_length;

	INITSTATUS(status, "LALShrinkCOMPLEX16FrequencySeries", FREQUENCYSERIESC);

	ASSERT(series != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(series->data != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(first + length <= series->data->length, status, LAL_RANGE_ERR, LAL_RANGE_MSG);
	new_length = XLALShrinkCOMPLEX16FrequencySeries (series, first, length);
	if(new_length != length) {
		XLALClearErrno();
		ABORT(status, LAL_FAIL_ERR, LAL_FAIL_MSG);
	}
	RETURN(status);
}


COMPLEX16FrequencySeries *XLALAddCOMPLEX16FrequencySeries (
	COMPLEX16FrequencySeries *arg1,
	const COMPLEX16FrequencySeries *arg2
)
{
	static const char *func = "XLALAddCOMPLEX16FrequencySeries";
	int offset = (arg2->f0 - arg1->f0) / arg1->deltaF;
	REAL8 ratio = XLALUnitRatio(&arg1->sampleUnits, &arg2->sampleUnits);
	unsigned int i;

	/* make sure arguments are compatible */
	if(XLALGPSCmp(&arg1->epoch, &arg2->epoch) || (arg1->deltaF != arg2->deltaF) || XLALIsREAL8FailNaN(ratio))
		XLAL_ERROR_NULL(func, XLAL_EDATA);
	/* FIXME: generalize to relax this requirement */
	if((arg2->f0 < arg1->f0) || (offset + arg2->data->length > arg1->data->length))
		XLAL_ERROR_NULL(func, XLAL_EBADLEN);
	
	/* add arg2 to arg1, adjusting the units */
	for(i = 0; i < arg2->data->length; i++) {
		arg1->data->data[offset + i].re += arg2->data->data[i].re / ratio;
		arg1->data->data[offset + i].im += arg2->data->data[i].im / ratio;
		
	}

	return(arg1);
}


COMPLEX16FrequencySeries *XLALSubtractCOMPLEX16FrequencySeries (
	COMPLEX16FrequencySeries *arg1,
	const COMPLEX16FrequencySeries *arg2
)
{
	static const char *func = "XLALAddCOMPLEX16FrequencySeries";
	int offset = (arg2->f0 - arg1->f0) / arg1->deltaF;
	REAL8 ratio = XLALUnitRatio(&arg1->sampleUnits, &arg2->sampleUnits);
	unsigned int i;

	/* make sure arguments are compatible */
	if(XLALGPSCmp(&arg1->epoch, &arg2->epoch) || (arg1->deltaF != arg2->deltaF) || XLALIsREAL8FailNaN(ratio))
		XLAL_ERROR_NULL(func, XLAL_EDATA);
	/* FIXME: generalize to relax this requirement */
	if((arg2->f0 < arg1->f0) || (offset + arg2->data->length > arg1->data->length))
		XLAL_ERROR_NULL(func, XLAL_EBADLEN);
	
	/* subtract arg2 from arg1, adjusting the units */
	for(i = 0; i < arg2->data->length; i++) {
		arg1->data->data[offset + i].re -= arg2->data->data[i].re / ratio;
		arg1->data->data[offset + i].im -= arg2->data->data[i].im / ratio;
		
	}

	return(arg1);
}


COMPLEX16FrequencySeries *XLALConjugateCOMPLEX16FrequencySeries (
	COMPLEX16FrequencySeries *series
)
{
	XLALConjugateCOMPLEX16Sequence (series->data);
	return(series);
}



COMPLEX16FrequencySeries *XLALMultiplyCOMPLEX16FrequencySeries (
	COMPLEX16FrequencySeries *arg1,
	const COMPLEX16FrequencySeries *arg2
)
{
	static const char *func = "XLALMultiplyCOMPLEX16FrequencySeries";
	int offset = (arg2->f0 - arg1->f0) / arg1->deltaF;
	REAL8 ratio = XLALUnitRatio(&arg1->sampleUnits, &arg2->sampleUnits);
	unsigned int i;

	/* make sure arguments are compatible */
	if(XLALGPSCmp(&arg1->epoch, &arg2->epoch) || (arg1->deltaF != arg2->deltaF) || XLALIsREAL8FailNaN(ratio))
		XLAL_ERROR_NULL(func, XLAL_EDATA);
	/* FIXME: generalize to relax this requirement */
	if((arg2->f0 < arg1->f0) || (offset + arg2->data->length > arg1->data->length))
		XLAL_ERROR_NULL(func, XLAL_EBADLEN);
	
	/* multiply arg2 by arg1, adjusting the units */
	for(i = 0; i < arg2->data->length; i++) {
		REAL8 re = arg2->data->data[i].re / ratio;
		REAL8 im = arg2->data->data[i].im / ratio;
		arg1->data->data[offset + i].re = arg1->data->data[offset + i].re * re - arg1->data->data[offset + i].im * im;
		arg1->data->data[offset + i].im = arg1->data->data[offset + i].re * im + arg1->data->data[offset + i].im * re;
		
	}

	return(arg1);
}






void XLALDestroyINT2FrequencySeries (
	INT2FrequencySeries *series
)
{
	if(series)
		XLALDestroyINT2Sequence (series->data);
	LALFree(series);
}


void LALDestroyINT2FrequencySeries (
	LALStatus *status,
	INT2FrequencySeries *series
)
{
	INITSTATUS(status, "LALDestroyINT2FrequencySeries", FREQUENCYSERIESC);
	XLALDestroyINT2FrequencySeries (series);
	RETURN(status);
}


INT2FrequencySeries *XLALCreateINT2FrequencySeries (
	const CHAR *name,
	const LIGOTimeGPS *epoch,
	REAL8 f0,
	REAL8 deltaF,
	const LALUnit *sampleUnits,
	size_t length
)
{
	static const char *func = "XLALCreateINT2FrequencySeries";
	INT2FrequencySeries *new;
	INT2Sequence *sequence;

	new = LALMalloc(sizeof(*new));
	sequence = XLALCreateINT2Sequence (length);
	if(!new || !sequence) {
		LALFree(new);
		XLALDestroyINT2Sequence (sequence);
		XLAL_ERROR_NULL(func, XLAL_ENOMEM);
	}

	if(name)
		strncpy(new->name, name, LALNameLength);
	else
		new->name[0] = '\0';
	new->epoch = *epoch;
	new->f0 = f0;
	new->deltaF = deltaF;
	new->sampleUnits = *sampleUnits;
	new->data = sequence;

	return(new);
}


void LALCreateINT2FrequencySeries (
	LALStatus *status,
	INT2FrequencySeries **output,
	const CHAR *name,
	LIGOTimeGPS epoch,
	REAL8 f0,
	REAL8 deltaF,
	LALUnit sampleUnits,
	size_t length
)
{
	INITSTATUS(status, "LALCreateINT2FrequencySeries", FREQUENCYSERIESC);
	ASSERT(output != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	*output = XLALCreateINT2FrequencySeries (name, &epoch, f0, deltaF, &sampleUnits, length);
	if(*output == NULL) {
		XLALClearErrno();
		ABORT(status, LAL_NOMEM_ERR, LAL_NOMEM_MSG);
	}
	RETURN(status);
}


INT2FrequencySeries *XLALCutINT2FrequencySeries (
	const INT2FrequencySeries *series,
	size_t first,
	size_t length
)
{
	static const char *func = "XLALCutINT2FrequencySeries";
	INT2FrequencySeries *new;
	INT2Sequence *sequence;

	if(!series || !series->data)
		return(NULL);

	new = LALMalloc(sizeof(*new));
	sequence = XLALCutINT2Sequence (series->data, first, length);
	if(!new || !sequence) {
		LALFree(new);
		XLALDestroyINT2Sequence (sequence);
		XLAL_ERROR_NULL(func, XLAL_ENOMEM);
	}

	*new = *series;
	new->data = sequence;
	new->f0 += first * new->deltaF;

	return(new);
}


size_t XLALResizeINT2FrequencySeries (
	INT2FrequencySeries *series,
	int first,
	size_t length
)
{
	if(!series)
		return(0);

	series->f0 += first * series->deltaF;
	return(XLALResizeINT2Sequence (series->data, first, length));
}


size_t XLALShrinkINT2FrequencySeries (
	INT2FrequencySeries *series,
	size_t first,
	size_t length
)
{
	return(XLALResizeINT2FrequencySeries (series, first, length));
}


void LALShrinkINT2FrequencySeries (
	LALStatus *status,
	INT2FrequencySeries *series,
	size_t first,
	size_t length
)
{
	size_t new_length;

	INITSTATUS(status, "LALShrinkINT2FrequencySeries", FREQUENCYSERIESC);

	ASSERT(series != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(series->data != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(first + length <= series->data->length, status, LAL_RANGE_ERR, LAL_RANGE_MSG);
	new_length = XLALShrinkINT2FrequencySeries (series, first, length);
	if(new_length != length) {
		XLALClearErrno();
		ABORT(status, LAL_FAIL_ERR, LAL_FAIL_MSG);
	}
	RETURN(status);
}


INT2FrequencySeries *XLALAddINT2FrequencySeries (
	INT2FrequencySeries *arg1,
	const INT2FrequencySeries *arg2
)
{
	static const char *func = "XLALAddINT2FrequencySeries";
	int offset = (arg2->f0 - arg1->f0) / arg1->deltaF;
	REAL8 ratio = XLALUnitRatio(&arg1->sampleUnits, &arg2->sampleUnits);
	unsigned int i;

	/* make sure arguments are compatible */
	if(XLALGPSCmp(&arg1->epoch, &arg2->epoch) || (arg1->deltaF != arg2->deltaF) || XLALIsREAL8FailNaN(ratio))
		XLAL_ERROR_NULL(func, XLAL_EDATA);
	/* FIXME: generalize to relax this requirement */
	if((arg2->f0 < arg1->f0) || (offset + arg2->data->length > arg1->data->length))
		XLAL_ERROR_NULL(func, XLAL_EBADLEN);
	
	/* add arg2 to arg1, adjusting the units */
	for(i = 0; i < arg2->data->length; i++) {
		arg1->data->data[offset + i] += arg2->data->data[i] / ratio;
	}

	return(arg1);
}


INT2FrequencySeries *XLALSubtractINT2FrequencySeries (
	INT2FrequencySeries *arg1,
	const INT2FrequencySeries *arg2
)
{
	static const char *func = "XLALAddINT2FrequencySeries";
	int offset = (arg2->f0 - arg1->f0) / arg1->deltaF;
	REAL8 ratio = XLALUnitRatio(&arg1->sampleUnits, &arg2->sampleUnits);
	unsigned int i;

	/* make sure arguments are compatible */
	if(XLALGPSCmp(&arg1->epoch, &arg2->epoch) || (arg1->deltaF != arg2->deltaF) || XLALIsREAL8FailNaN(ratio))
		XLAL_ERROR_NULL(func, XLAL_EDATA);
	/* FIXME: generalize to relax this requirement */
	if((arg2->f0 < arg1->f0) || (offset + arg2->data->length > arg1->data->length))
		XLAL_ERROR_NULL(func, XLAL_EBADLEN);
	
	/* subtract arg2 from arg1, adjusting the units */
	for(i = 0; i < arg2->data->length; i++) {
		arg1->data->data[offset + i] -= arg2->data->data[i] / ratio;
	}

	return(arg1);
}





INT2FrequencySeries *XLALMultiplyINT2FrequencySeries (
	INT2FrequencySeries *arg1,
	const INT2FrequencySeries *arg2
)
{
	static const char *func = "XLALMultiplyINT2FrequencySeries";
	int offset = (arg2->f0 - arg1->f0) / arg1->deltaF;
	REAL8 ratio = XLALUnitRatio(&arg1->sampleUnits, &arg2->sampleUnits);
	unsigned int i;

	/* make sure arguments are compatible */
	if(XLALGPSCmp(&arg1->epoch, &arg2->epoch) || (arg1->deltaF != arg2->deltaF) || XLALIsREAL8FailNaN(ratio))
		XLAL_ERROR_NULL(func, XLAL_EDATA);
	/* FIXME: generalize to relax this requirement */
	if((arg2->f0 < arg1->f0) || (offset + arg2->data->length > arg1->data->length))
		XLAL_ERROR_NULL(func, XLAL_EBADLEN);
	
	/* multiply arg2 by arg1, adjusting the units */
	for(i = 0; i < arg2->data->length; i++) {
		arg1->data->data[offset + i] *= arg2->data->data[i] / ratio;
	}

	return(arg1);
}






void XLALDestroyUINT2FrequencySeries (
	UINT2FrequencySeries *series
)
{
	if(series)
		XLALDestroyUINT2Sequence (series->data);
	LALFree(series);
}


void LALDestroyUINT2FrequencySeries (
	LALStatus *status,
	UINT2FrequencySeries *series
)
{
	INITSTATUS(status, "LALDestroyUINT2FrequencySeries", FREQUENCYSERIESC);
	XLALDestroyUINT2FrequencySeries (series);
	RETURN(status);
}


UINT2FrequencySeries *XLALCreateUINT2FrequencySeries (
	const CHAR *name,
	const LIGOTimeGPS *epoch,
	REAL8 f0,
	REAL8 deltaF,
	const LALUnit *sampleUnits,
	size_t length
)
{
	static const char *func = "XLALCreateUINT2FrequencySeries";
	UINT2FrequencySeries *new;
	UINT2Sequence *sequence;

	new = LALMalloc(sizeof(*new));
	sequence = XLALCreateUINT2Sequence (length);
	if(!new || !sequence) {
		LALFree(new);
		XLALDestroyUINT2Sequence (sequence);
		XLAL_ERROR_NULL(func, XLAL_ENOMEM);
	}

	if(name)
		strncpy(new->name, name, LALNameLength);
	else
		new->name[0] = '\0';
	new->epoch = *epoch;
	new->f0 = f0;
	new->deltaF = deltaF;
	new->sampleUnits = *sampleUnits;
	new->data = sequence;

	return(new);
}


void LALCreateUINT2FrequencySeries (
	LALStatus *status,
	UINT2FrequencySeries **output,
	const CHAR *name,
	LIGOTimeGPS epoch,
	REAL8 f0,
	REAL8 deltaF,
	LALUnit sampleUnits,
	size_t length
)
{
	INITSTATUS(status, "LALCreateUINT2FrequencySeries", FREQUENCYSERIESC);
	ASSERT(output != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	*output = XLALCreateUINT2FrequencySeries (name, &epoch, f0, deltaF, &sampleUnits, length);
	if(*output == NULL) {
		XLALClearErrno();
		ABORT(status, LAL_NOMEM_ERR, LAL_NOMEM_MSG);
	}
	RETURN(status);
}


UINT2FrequencySeries *XLALCutUINT2FrequencySeries (
	const UINT2FrequencySeries *series,
	size_t first,
	size_t length
)
{
	static const char *func = "XLALCutUINT2FrequencySeries";
	UINT2FrequencySeries *new;
	UINT2Sequence *sequence;

	if(!series || !series->data)
		return(NULL);

	new = LALMalloc(sizeof(*new));
	sequence = XLALCutUINT2Sequence (series->data, first, length);
	if(!new || !sequence) {
		LALFree(new);
		XLALDestroyUINT2Sequence (sequence);
		XLAL_ERROR_NULL(func, XLAL_ENOMEM);
	}

	*new = *series;
	new->data = sequence;
	new->f0 += first * new->deltaF;

	return(new);
}


size_t XLALResizeUINT2FrequencySeries (
	UINT2FrequencySeries *series,
	int first,
	size_t length
)
{
	if(!series)
		return(0);

	series->f0 += first * series->deltaF;
	return(XLALResizeUINT2Sequence (series->data, first, length));
}


size_t XLALShrinkUINT2FrequencySeries (
	UINT2FrequencySeries *series,
	size_t first,
	size_t length
)
{
	return(XLALResizeUINT2FrequencySeries (series, first, length));
}


void LALShrinkUINT2FrequencySeries (
	LALStatus *status,
	UINT2FrequencySeries *series,
	size_t first,
	size_t length
)
{
	size_t new_length;

	INITSTATUS(status, "LALShrinkUINT2FrequencySeries", FREQUENCYSERIESC);

	ASSERT(series != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(series->data != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(first + length <= series->data->length, status, LAL_RANGE_ERR, LAL_RANGE_MSG);
	new_length = XLALShrinkUINT2FrequencySeries (series, first, length);
	if(new_length != length) {
		XLALClearErrno();
		ABORT(status, LAL_FAIL_ERR, LAL_FAIL_MSG);
	}
	RETURN(status);
}


UINT2FrequencySeries *XLALAddUINT2FrequencySeries (
	UINT2FrequencySeries *arg1,
	const UINT2FrequencySeries *arg2
)
{
	static const char *func = "XLALAddUINT2FrequencySeries";
	int offset = (arg2->f0 - arg1->f0) / arg1->deltaF;
	REAL8 ratio = XLALUnitRatio(&arg1->sampleUnits, &arg2->sampleUnits);
	unsigned int i;

	/* make sure arguments are compatible */
	if(XLALGPSCmp(&arg1->epoch, &arg2->epoch) || (arg1->deltaF != arg2->deltaF) || XLALIsREAL8FailNaN(ratio))
		XLAL_ERROR_NULL(func, XLAL_EDATA);
	/* FIXME: generalize to relax this requirement */
	if((arg2->f0 < arg1->f0) || (offset + arg2->data->length > arg1->data->length))
		XLAL_ERROR_NULL(func, XLAL_EBADLEN);
	
	/* add arg2 to arg1, adjusting the units */
	for(i = 0; i < arg2->data->length; i++) {
		arg1->data->data[offset + i] += arg2->data->data[i] / ratio;
	}

	return(arg1);
}


UINT2FrequencySeries *XLALSubtractUINT2FrequencySeries (
	UINT2FrequencySeries *arg1,
	const UINT2FrequencySeries *arg2
)
{
	static const char *func = "XLALAddUINT2FrequencySeries";
	int offset = (arg2->f0 - arg1->f0) / arg1->deltaF;
	REAL8 ratio = XLALUnitRatio(&arg1->sampleUnits, &arg2->sampleUnits);
	unsigned int i;

	/* make sure arguments are compatible */
	if(XLALGPSCmp(&arg1->epoch, &arg2->epoch) || (arg1->deltaF != arg2->deltaF) || XLALIsREAL8FailNaN(ratio))
		XLAL_ERROR_NULL(func, XLAL_EDATA);
	/* FIXME: generalize to relax this requirement */
	if((arg2->f0 < arg1->f0) || (offset + arg2->data->length > arg1->data->length))
		XLAL_ERROR_NULL(func, XLAL_EBADLEN);
	
	/* subtract arg2 from arg1, adjusting the units */
	for(i = 0; i < arg2->data->length; i++) {
		arg1->data->data[offset + i] -= arg2->data->data[i] / ratio;
	}

	return(arg1);
}





UINT2FrequencySeries *XLALMultiplyUINT2FrequencySeries (
	UINT2FrequencySeries *arg1,
	const UINT2FrequencySeries *arg2
)
{
	static const char *func = "XLALMultiplyUINT2FrequencySeries";
	int offset = (arg2->f0 - arg1->f0) / arg1->deltaF;
	REAL8 ratio = XLALUnitRatio(&arg1->sampleUnits, &arg2->sampleUnits);
	unsigned int i;

	/* make sure arguments are compatible */
	if(XLALGPSCmp(&arg1->epoch, &arg2->epoch) || (arg1->deltaF != arg2->deltaF) || XLALIsREAL8FailNaN(ratio))
		XLAL_ERROR_NULL(func, XLAL_EDATA);
	/* FIXME: generalize to relax this requirement */
	if((arg2->f0 < arg1->f0) || (offset + arg2->data->length > arg1->data->length))
		XLAL_ERROR_NULL(func, XLAL_EBADLEN);
	
	/* multiply arg2 by arg1, adjusting the units */
	for(i = 0; i < arg2->data->length; i++) {
		arg1->data->data[offset + i] *= arg2->data->data[i] / ratio;
	}

	return(arg1);
}






void XLALDestroyINT4FrequencySeries (
	INT4FrequencySeries *series
)
{
	if(series)
		XLALDestroyINT4Sequence (series->data);
	LALFree(series);
}


void LALDestroyINT4FrequencySeries (
	LALStatus *status,
	INT4FrequencySeries *series
)
{
	INITSTATUS(status, "LALDestroyINT4FrequencySeries", FREQUENCYSERIESC);
	XLALDestroyINT4FrequencySeries (series);
	RETURN(status);
}


INT4FrequencySeries *XLALCreateINT4FrequencySeries (
	const CHAR *name,
	const LIGOTimeGPS *epoch,
	REAL8 f0,
	REAL8 deltaF,
	const LALUnit *sampleUnits,
	size_t length
)
{
	static const char *func = "XLALCreateINT4FrequencySeries";
	INT4FrequencySeries *new;
	INT4Sequence *sequence;

	new = LALMalloc(sizeof(*new));
	sequence = XLALCreateINT4Sequence (length);
	if(!new || !sequence) {
		LALFree(new);
		XLALDestroyINT4Sequence (sequence);
		XLAL_ERROR_NULL(func, XLAL_ENOMEM);
	}

	if(name)
		strncpy(new->name, name, LALNameLength);
	else
		new->name[0] = '\0';
	new->epoch = *epoch;
	new->f0 = f0;
	new->deltaF = deltaF;
	new->sampleUnits = *sampleUnits;
	new->data = sequence;

	return(new);
}


void LALCreateINT4FrequencySeries (
	LALStatus *status,
	INT4FrequencySeries **output,
	const CHAR *name,
	LIGOTimeGPS epoch,
	REAL8 f0,
	REAL8 deltaF,
	LALUnit sampleUnits,
	size_t length
)
{
	INITSTATUS(status, "LALCreateINT4FrequencySeries", FREQUENCYSERIESC);
	ASSERT(output != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	*output = XLALCreateINT4FrequencySeries (name, &epoch, f0, deltaF, &sampleUnits, length);
	if(*output == NULL) {
		XLALClearErrno();
		ABORT(status, LAL_NOMEM_ERR, LAL_NOMEM_MSG);
	}
	RETURN(status);
}


INT4FrequencySeries *XLALCutINT4FrequencySeries (
	const INT4FrequencySeries *series,
	size_t first,
	size_t length
)
{
	static const char *func = "XLALCutINT4FrequencySeries";
	INT4FrequencySeries *new;
	INT4Sequence *sequence;

	if(!series || !series->data)
		return(NULL);

	new = LALMalloc(sizeof(*new));
	sequence = XLALCutINT4Sequence (series->data, first, length);
	if(!new || !sequence) {
		LALFree(new);
		XLALDestroyINT4Sequence (sequence);
		XLAL_ERROR_NULL(func, XLAL_ENOMEM);
	}

	*new = *series;
	new->data = sequence;
	new->f0 += first * new->deltaF;

	return(new);
}


size_t XLALResizeINT4FrequencySeries (
	INT4FrequencySeries *series,
	int first,
	size_t length
)
{
	if(!series)
		return(0);

	series->f0 += first * series->deltaF;
	return(XLALResizeINT4Sequence (series->data, first, length));
}


size_t XLALShrinkINT4FrequencySeries (
	INT4FrequencySeries *series,
	size_t first,
	size_t length
)
{
	return(XLALResizeINT4FrequencySeries (series, first, length));
}


void LALShrinkINT4FrequencySeries (
	LALStatus *status,
	INT4FrequencySeries *series,
	size_t first,
	size_t length
)
{
	size_t new_length;

	INITSTATUS(status, "LALShrinkINT4FrequencySeries", FREQUENCYSERIESC);

	ASSERT(series != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(series->data != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(first + length <= series->data->length, status, LAL_RANGE_ERR, LAL_RANGE_MSG);
	new_length = XLALShrinkINT4FrequencySeries (series, first, length);
	if(new_length != length) {
		XLALClearErrno();
		ABORT(status, LAL_FAIL_ERR, LAL_FAIL_MSG);
	}
	RETURN(status);
}


INT4FrequencySeries *XLALAddINT4FrequencySeries (
	INT4FrequencySeries *arg1,
	const INT4FrequencySeries *arg2
)
{
	static const char *func = "XLALAddINT4FrequencySeries";
	int offset = (arg2->f0 - arg1->f0) / arg1->deltaF;
	REAL8 ratio = XLALUnitRatio(&arg1->sampleUnits, &arg2->sampleUnits);
	unsigned int i;

	/* make sure arguments are compatible */
	if(XLALGPSCmp(&arg1->epoch, &arg2->epoch) || (arg1->deltaF != arg2->deltaF) || XLALIsREAL8FailNaN(ratio))
		XLAL_ERROR_NULL(func, XLAL_EDATA);
	/* FIXME: generalize to relax this requirement */
	if((arg2->f0 < arg1->f0) || (offset + arg2->data->length > arg1->data->length))
		XLAL_ERROR_NULL(func, XLAL_EBADLEN);
	
	/* add arg2 to arg1, adjusting the units */
	for(i = 0; i < arg2->data->length; i++) {
		arg1->data->data[offset + i] += arg2->data->data[i] / ratio;
	}

	return(arg1);
}


INT4FrequencySeries *XLALSubtractINT4FrequencySeries (
	INT4FrequencySeries *arg1,
	const INT4FrequencySeries *arg2
)
{
	static const char *func = "XLALAddINT4FrequencySeries";
	int offset = (arg2->f0 - arg1->f0) / arg1->deltaF;
	REAL8 ratio = XLALUnitRatio(&arg1->sampleUnits, &arg2->sampleUnits);
	unsigned int i;

	/* make sure arguments are compatible */
	if(XLALGPSCmp(&arg1->epoch, &arg2->epoch) || (arg1->deltaF != arg2->deltaF) || XLALIsREAL8FailNaN(ratio))
		XLAL_ERROR_NULL(func, XLAL_EDATA);
	/* FIXME: generalize to relax this requirement */
	if((arg2->f0 < arg1->f0) || (offset + arg2->data->length > arg1->data->length))
		XLAL_ERROR_NULL(func, XLAL_EBADLEN);
	
	/* subtract arg2 from arg1, adjusting the units */
	for(i = 0; i < arg2->data->length; i++) {
		arg1->data->data[offset + i] -= arg2->data->data[i] / ratio;
	}

	return(arg1);
}





INT4FrequencySeries *XLALMultiplyINT4FrequencySeries (
	INT4FrequencySeries *arg1,
	const INT4FrequencySeries *arg2
)
{
	static const char *func = "XLALMultiplyINT4FrequencySeries";
	int offset = (arg2->f0 - arg1->f0) / arg1->deltaF;
	REAL8 ratio = XLALUnitRatio(&arg1->sampleUnits, &arg2->sampleUnits);
	unsigned int i;

	/* make sure arguments are compatible */
	if(XLALGPSCmp(&arg1->epoch, &arg2->epoch) || (arg1->deltaF != arg2->deltaF) || XLALIsREAL8FailNaN(ratio))
		XLAL_ERROR_NULL(func, XLAL_EDATA);
	/* FIXME: generalize to relax this requirement */
	if((arg2->f0 < arg1->f0) || (offset + arg2->data->length > arg1->data->length))
		XLAL_ERROR_NULL(func, XLAL_EBADLEN);
	
	/* multiply arg2 by arg1, adjusting the units */
	for(i = 0; i < arg2->data->length; i++) {
		arg1->data->data[offset + i] *= arg2->data->data[i] / ratio;
	}

	return(arg1);
}






void XLALDestroyUINT4FrequencySeries (
	UINT4FrequencySeries *series
)
{
	if(series)
		XLALDestroyUINT4Sequence (series->data);
	LALFree(series);
}


void LALDestroyUINT4FrequencySeries (
	LALStatus *status,
	UINT4FrequencySeries *series
)
{
	INITSTATUS(status, "LALDestroyUINT4FrequencySeries", FREQUENCYSERIESC);
	XLALDestroyUINT4FrequencySeries (series);
	RETURN(status);
}


UINT4FrequencySeries *XLALCreateUINT4FrequencySeries (
	const CHAR *name,
	const LIGOTimeGPS *epoch,
	REAL8 f0,
	REAL8 deltaF,
	const LALUnit *sampleUnits,
	size_t length
)
{
	static const char *func = "XLALCreateUINT4FrequencySeries";
	UINT4FrequencySeries *new;
	UINT4Sequence *sequence;

	new = LALMalloc(sizeof(*new));
	sequence = XLALCreateUINT4Sequence (length);
	if(!new || !sequence) {
		LALFree(new);
		XLALDestroyUINT4Sequence (sequence);
		XLAL_ERROR_NULL(func, XLAL_ENOMEM);
	}

	if(name)
		strncpy(new->name, name, LALNameLength);
	else
		new->name[0] = '\0';
	new->epoch = *epoch;
	new->f0 = f0;
	new->deltaF = deltaF;
	new->sampleUnits = *sampleUnits;
	new->data = sequence;

	return(new);
}


void LALCreateUINT4FrequencySeries (
	LALStatus *status,
	UINT4FrequencySeries **output,
	const CHAR *name,
	LIGOTimeGPS epoch,
	REAL8 f0,
	REAL8 deltaF,
	LALUnit sampleUnits,
	size_t length
)
{
	INITSTATUS(status, "LALCreateUINT4FrequencySeries", FREQUENCYSERIESC);
	ASSERT(output != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	*output = XLALCreateUINT4FrequencySeries (name, &epoch, f0, deltaF, &sampleUnits, length);
	if(*output == NULL) {
		XLALClearErrno();
		ABORT(status, LAL_NOMEM_ERR, LAL_NOMEM_MSG);
	}
	RETURN(status);
}


UINT4FrequencySeries *XLALCutUINT4FrequencySeries (
	const UINT4FrequencySeries *series,
	size_t first,
	size_t length
)
{
	static const char *func = "XLALCutUINT4FrequencySeries";
	UINT4FrequencySeries *new;
	UINT4Sequence *sequence;

	if(!series || !series->data)
		return(NULL);

	new = LALMalloc(sizeof(*new));
	sequence = XLALCutUINT4Sequence (series->data, first, length);
	if(!new || !sequence) {
		LALFree(new);
		XLALDestroyUINT4Sequence (sequence);
		XLAL_ERROR_NULL(func, XLAL_ENOMEM);
	}

	*new = *series;
	new->data = sequence;
	new->f0 += first * new->deltaF;

	return(new);
}


size_t XLALResizeUINT4FrequencySeries (
	UINT4FrequencySeries *series,
	int first,
	size_t length
)
{
	if(!series)
		return(0);

	series->f0 += first * series->deltaF;
	return(XLALResizeUINT4Sequence (series->data, first, length));
}


size_t XLALShrinkUINT4FrequencySeries (
	UINT4FrequencySeries *series,
	size_t first,
	size_t length
)
{
	return(XLALResizeUINT4FrequencySeries (series, first, length));
}


void LALShrinkUINT4FrequencySeries (
	LALStatus *status,
	UINT4FrequencySeries *series,
	size_t first,
	size_t length
)
{
	size_t new_length;

	INITSTATUS(status, "LALShrinkUINT4FrequencySeries", FREQUENCYSERIESC);

	ASSERT(series != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(series->data != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(first + length <= series->data->length, status, LAL_RANGE_ERR, LAL_RANGE_MSG);
	new_length = XLALShrinkUINT4FrequencySeries (series, first, length);
	if(new_length != length) {
		XLALClearErrno();
		ABORT(status, LAL_FAIL_ERR, LAL_FAIL_MSG);
	}
	RETURN(status);
}


UINT4FrequencySeries *XLALAddUINT4FrequencySeries (
	UINT4FrequencySeries *arg1,
	const UINT4FrequencySeries *arg2
)
{
	static const char *func = "XLALAddUINT4FrequencySeries";
	int offset = (arg2->f0 - arg1->f0) / arg1->deltaF;
	REAL8 ratio = XLALUnitRatio(&arg1->sampleUnits, &arg2->sampleUnits);
	unsigned int i;

	/* make sure arguments are compatible */
	if(XLALGPSCmp(&arg1->epoch, &arg2->epoch) || (arg1->deltaF != arg2->deltaF) || XLALIsREAL8FailNaN(ratio))
		XLAL_ERROR_NULL(func, XLAL_EDATA);
	/* FIXME: generalize to relax this requirement */
	if((arg2->f0 < arg1->f0) || (offset + arg2->data->length > arg1->data->length))
		XLAL_ERROR_NULL(func, XLAL_EBADLEN);
	
	/* add arg2 to arg1, adjusting the units */
	for(i = 0; i < arg2->data->length; i++) {
		arg1->data->data[offset + i] += arg2->data->data[i] / ratio;
	}

	return(arg1);
}


UINT4FrequencySeries *XLALSubtractUINT4FrequencySeries (
	UINT4FrequencySeries *arg1,
	const UINT4FrequencySeries *arg2
)
{
	static const char *func = "XLALAddUINT4FrequencySeries";
	int offset = (arg2->f0 - arg1->f0) / arg1->deltaF;
	REAL8 ratio = XLALUnitRatio(&arg1->sampleUnits, &arg2->sampleUnits);
	unsigned int i;

	/* make sure arguments are compatible */
	if(XLALGPSCmp(&arg1->epoch, &arg2->epoch) || (arg1->deltaF != arg2->deltaF) || XLALIsREAL8FailNaN(ratio))
		XLAL_ERROR_NULL(func, XLAL_EDATA);
	/* FIXME: generalize to relax this requirement */
	if((arg2->f0 < arg1->f0) || (offset + arg2->data->length > arg1->data->length))
		XLAL_ERROR_NULL(func, XLAL_EBADLEN);
	
	/* subtract arg2 from arg1, adjusting the units */
	for(i = 0; i < arg2->data->length; i++) {
		arg1->data->data[offset + i] -= arg2->data->data[i] / ratio;
	}

	return(arg1);
}





UINT4FrequencySeries *XLALMultiplyUINT4FrequencySeries (
	UINT4FrequencySeries *arg1,
	const UINT4FrequencySeries *arg2
)
{
	static const char *func = "XLALMultiplyUINT4FrequencySeries";
	int offset = (arg2->f0 - arg1->f0) / arg1->deltaF;
	REAL8 ratio = XLALUnitRatio(&arg1->sampleUnits, &arg2->sampleUnits);
	unsigned int i;

	/* make sure arguments are compatible */
	if(XLALGPSCmp(&arg1->epoch, &arg2->epoch) || (arg1->deltaF != arg2->deltaF) || XLALIsREAL8FailNaN(ratio))
		XLAL_ERROR_NULL(func, XLAL_EDATA);
	/* FIXME: generalize to relax this requirement */
	if((arg2->f0 < arg1->f0) || (offset + arg2->data->length > arg1->data->length))
		XLAL_ERROR_NULL(func, XLAL_EBADLEN);
	
	/* multiply arg2 by arg1, adjusting the units */
	for(i = 0; i < arg2->data->length; i++) {
		arg1->data->data[offset + i] *= arg2->data->data[i] / ratio;
	}

	return(arg1);
}






void XLALDestroyINT8FrequencySeries (
	INT8FrequencySeries *series
)
{
	if(series)
		XLALDestroyINT8Sequence (series->data);
	LALFree(series);
}


void LALDestroyINT8FrequencySeries (
	LALStatus *status,
	INT8FrequencySeries *series
)
{
	INITSTATUS(status, "LALDestroyINT8FrequencySeries", FREQUENCYSERIESC);
	XLALDestroyINT8FrequencySeries (series);
	RETURN(status);
}


INT8FrequencySeries *XLALCreateINT8FrequencySeries (
	const CHAR *name,
	const LIGOTimeGPS *epoch,
	REAL8 f0,
	REAL8 deltaF,
	const LALUnit *sampleUnits,
	size_t length
)
{
	static const char *func = "XLALCreateINT8FrequencySeries";
	INT8FrequencySeries *new;
	INT8Sequence *sequence;

	new = LALMalloc(sizeof(*new));
	sequence = XLALCreateINT8Sequence (length);
	if(!new || !sequence) {
		LALFree(new);
		XLALDestroyINT8Sequence (sequence);
		XLAL_ERROR_NULL(func, XLAL_ENOMEM);
	}

	if(name)
		strncpy(new->name, name, LALNameLength);
	else
		new->name[0] = '\0';
	new->epoch = *epoch;
	new->f0 = f0;
	new->deltaF = deltaF;
	new->sampleUnits = *sampleUnits;
	new->data = sequence;

	return(new);
}


void LALCreateINT8FrequencySeries (
	LALStatus *status,
	INT8FrequencySeries **output,
	const CHAR *name,
	LIGOTimeGPS epoch,
	REAL8 f0,
	REAL8 deltaF,
	LALUnit sampleUnits,
	size_t length
)
{
	INITSTATUS(status, "LALCreateINT8FrequencySeries", FREQUENCYSERIESC);
	ASSERT(output != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	*output = XLALCreateINT8FrequencySeries (name, &epoch, f0, deltaF, &sampleUnits, length);
	if(*output == NULL) {
		XLALClearErrno();
		ABORT(status, LAL_NOMEM_ERR, LAL_NOMEM_MSG);
	}
	RETURN(status);
}


INT8FrequencySeries *XLALCutINT8FrequencySeries (
	const INT8FrequencySeries *series,
	size_t first,
	size_t length
)
{
	static const char *func = "XLALCutINT8FrequencySeries";
	INT8FrequencySeries *new;
	INT8Sequence *sequence;

	if(!series || !series->data)
		return(NULL);

	new = LALMalloc(sizeof(*new));
	sequence = XLALCutINT8Sequence (series->data, first, length);
	if(!new || !sequence) {
		LALFree(new);
		XLALDestroyINT8Sequence (sequence);
		XLAL_ERROR_NULL(func, XLAL_ENOMEM);
	}

	*new = *series;
	new->data = sequence;
	new->f0 += first * new->deltaF;

	return(new);
}


size_t XLALResizeINT8FrequencySeries (
	INT8FrequencySeries *series,
	int first,
	size_t length
)
{
	if(!series)
		return(0);

	series->f0 += first * series->deltaF;
	return(XLALResizeINT8Sequence (series->data, first, length));
}


size_t XLALShrinkINT8FrequencySeries (
	INT8FrequencySeries *series,
	size_t first,
	size_t length
)
{
	return(XLALResizeINT8FrequencySeries (series, first, length));
}


void LALShrinkINT8FrequencySeries (
	LALStatus *status,
	INT8FrequencySeries *series,
	size_t first,
	size_t length
)
{
	size_t new_length;

	INITSTATUS(status, "LALShrinkINT8FrequencySeries", FREQUENCYSERIESC);

	ASSERT(series != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(series->data != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(first + length <= series->data->length, status, LAL_RANGE_ERR, LAL_RANGE_MSG);
	new_length = XLALShrinkINT8FrequencySeries (series, first, length);
	if(new_length != length) {
		XLALClearErrno();
		ABORT(status, LAL_FAIL_ERR, LAL_FAIL_MSG);
	}
	RETURN(status);
}


INT8FrequencySeries *XLALAddINT8FrequencySeries (
	INT8FrequencySeries *arg1,
	const INT8FrequencySeries *arg2
)
{
	static const char *func = "XLALAddINT8FrequencySeries";
	int offset = (arg2->f0 - arg1->f0) / arg1->deltaF;
	REAL8 ratio = XLALUnitRatio(&arg1->sampleUnits, &arg2->sampleUnits);
	unsigned int i;

	/* make sure arguments are compatible */
	if(XLALGPSCmp(&arg1->epoch, &arg2->epoch) || (arg1->deltaF != arg2->deltaF) || XLALIsREAL8FailNaN(ratio))
		XLAL_ERROR_NULL(func, XLAL_EDATA);
	/* FIXME: generalize to relax this requirement */
	if((arg2->f0 < arg1->f0) || (offset + arg2->data->length > arg1->data->length))
		XLAL_ERROR_NULL(func, XLAL_EBADLEN);
	
	/* add arg2 to arg1, adjusting the units */
	for(i = 0; i < arg2->data->length; i++) {
		arg1->data->data[offset + i] += arg2->data->data[i] / ratio;
	}

	return(arg1);
}


INT8FrequencySeries *XLALSubtractINT8FrequencySeries (
	INT8FrequencySeries *arg1,
	const INT8FrequencySeries *arg2
)
{
	static const char *func = "XLALAddINT8FrequencySeries";
	int offset = (arg2->f0 - arg1->f0) / arg1->deltaF;
	REAL8 ratio = XLALUnitRatio(&arg1->sampleUnits, &arg2->sampleUnits);
	unsigned int i;

	/* make sure arguments are compatible */
	if(XLALGPSCmp(&arg1->epoch, &arg2->epoch) || (arg1->deltaF != arg2->deltaF) || XLALIsREAL8FailNaN(ratio))
		XLAL_ERROR_NULL(func, XLAL_EDATA);
	/* FIXME: generalize to relax this requirement */
	if((arg2->f0 < arg1->f0) || (offset + arg2->data->length > arg1->data->length))
		XLAL_ERROR_NULL(func, XLAL_EBADLEN);
	
	/* subtract arg2 from arg1, adjusting the units */
	for(i = 0; i < arg2->data->length; i++) {
		arg1->data->data[offset + i] -= arg2->data->data[i] / ratio;
	}

	return(arg1);
}





INT8FrequencySeries *XLALMultiplyINT8FrequencySeries (
	INT8FrequencySeries *arg1,
	const INT8FrequencySeries *arg2
)
{
	static const char *func = "XLALMultiplyINT8FrequencySeries";
	int offset = (arg2->f0 - arg1->f0) / arg1->deltaF;
	REAL8 ratio = XLALUnitRatio(&arg1->sampleUnits, &arg2->sampleUnits);
	unsigned int i;

	/* make sure arguments are compatible */
	if(XLALGPSCmp(&arg1->epoch, &arg2->epoch) || (arg1->deltaF != arg2->deltaF) || XLALIsREAL8FailNaN(ratio))
		XLAL_ERROR_NULL(func, XLAL_EDATA);
	/* FIXME: generalize to relax this requirement */
	if((arg2->f0 < arg1->f0) || (offset + arg2->data->length > arg1->data->length))
		XLAL_ERROR_NULL(func, XLAL_EBADLEN);
	
	/* multiply arg2 by arg1, adjusting the units */
	for(i = 0; i < arg2->data->length; i++) {
		arg1->data->data[offset + i] *= arg2->data->data[i] / ratio;
	}

	return(arg1);
}






void XLALDestroyUINT8FrequencySeries (
	UINT8FrequencySeries *series
)
{
	if(series)
		XLALDestroyUINT8Sequence (series->data);
	LALFree(series);
}


void LALDestroyUINT8FrequencySeries (
	LALStatus *status,
	UINT8FrequencySeries *series
)
{
	INITSTATUS(status, "LALDestroyUINT8FrequencySeries", FREQUENCYSERIESC);
	XLALDestroyUINT8FrequencySeries (series);
	RETURN(status);
}


UINT8FrequencySeries *XLALCreateUINT8FrequencySeries (
	const CHAR *name,
	const LIGOTimeGPS *epoch,
	REAL8 f0,
	REAL8 deltaF,
	const LALUnit *sampleUnits,
	size_t length
)
{
	static const char *func = "XLALCreateUINT8FrequencySeries";
	UINT8FrequencySeries *new;
	UINT8Sequence *sequence;

	new = LALMalloc(sizeof(*new));
	sequence = XLALCreateUINT8Sequence (length);
	if(!new || !sequence) {
		LALFree(new);
		XLALDestroyUINT8Sequence (sequence);
		XLAL_ERROR_NULL(func, XLAL_ENOMEM);
	}

	if(name)
		strncpy(new->name, name, LALNameLength);
	else
		new->name[0] = '\0';
	new->epoch = *epoch;
	new->f0 = f0;
	new->deltaF = deltaF;
	new->sampleUnits = *sampleUnits;
	new->data = sequence;

	return(new);
}


void LALCreateUINT8FrequencySeries (
	LALStatus *status,
	UINT8FrequencySeries **output,
	const CHAR *name,
	LIGOTimeGPS epoch,
	REAL8 f0,
	REAL8 deltaF,
	LALUnit sampleUnits,
	size_t length
)
{
	INITSTATUS(status, "LALCreateUINT8FrequencySeries", FREQUENCYSERIESC);
	ASSERT(output != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	*output = XLALCreateUINT8FrequencySeries (name, &epoch, f0, deltaF, &sampleUnits, length);
	if(*output == NULL) {
		XLALClearErrno();
		ABORT(status, LAL_NOMEM_ERR, LAL_NOMEM_MSG);
	}
	RETURN(status);
}


UINT8FrequencySeries *XLALCutUINT8FrequencySeries (
	const UINT8FrequencySeries *series,
	size_t first,
	size_t length
)
{
	static const char *func = "XLALCutUINT8FrequencySeries";
	UINT8FrequencySeries *new;
	UINT8Sequence *sequence;

	if(!series || !series->data)
		return(NULL);

	new = LALMalloc(sizeof(*new));
	sequence = XLALCutUINT8Sequence (series->data, first, length);
	if(!new || !sequence) {
		LALFree(new);
		XLALDestroyUINT8Sequence (sequence);
		XLAL_ERROR_NULL(func, XLAL_ENOMEM);
	}

	*new = *series;
	new->data = sequence;
	new->f0 += first * new->deltaF;

	return(new);
}


size_t XLALResizeUINT8FrequencySeries (
	UINT8FrequencySeries *series,
	int first,
	size_t length
)
{
	if(!series)
		return(0);

	series->f0 += first * series->deltaF;
	return(XLALResizeUINT8Sequence (series->data, first, length));
}


size_t XLALShrinkUINT8FrequencySeries (
	UINT8FrequencySeries *series,
	size_t first,
	size_t length
)
{
	return(XLALResizeUINT8FrequencySeries (series, first, length));
}


void LALShrinkUINT8FrequencySeries (
	LALStatus *status,
	UINT8FrequencySeries *series,
	size_t first,
	size_t length
)
{
	size_t new_length;

	INITSTATUS(status, "LALShrinkUINT8FrequencySeries", FREQUENCYSERIESC);

	ASSERT(series != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(series->data != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(first + length <= series->data->length, status, LAL_RANGE_ERR, LAL_RANGE_MSG);
	new_length = XLALShrinkUINT8FrequencySeries (series, first, length);
	if(new_length != length) {
		XLALClearErrno();
		ABORT(status, LAL_FAIL_ERR, LAL_FAIL_MSG);
	}
	RETURN(status);
}


UINT8FrequencySeries *XLALAddUINT8FrequencySeries (
	UINT8FrequencySeries *arg1,
	const UINT8FrequencySeries *arg2
)
{
	static const char *func = "XLALAddUINT8FrequencySeries";
	int offset = (arg2->f0 - arg1->f0) / arg1->deltaF;
	REAL8 ratio = XLALUnitRatio(&arg1->sampleUnits, &arg2->sampleUnits);
	unsigned int i;

	/* make sure arguments are compatible */
	if(XLALGPSCmp(&arg1->epoch, &arg2->epoch) || (arg1->deltaF != arg2->deltaF) || XLALIsREAL8FailNaN(ratio))
		XLAL_ERROR_NULL(func, XLAL_EDATA);
	/* FIXME: generalize to relax this requirement */
	if((arg2->f0 < arg1->f0) || (offset + arg2->data->length > arg1->data->length))
		XLAL_ERROR_NULL(func, XLAL_EBADLEN);
	
	/* add arg2 to arg1, adjusting the units */
	for(i = 0; i < arg2->data->length; i++) {
		arg1->data->data[offset + i] += arg2->data->data[i] / ratio;
	}

	return(arg1);
}


UINT8FrequencySeries *XLALSubtractUINT8FrequencySeries (
	UINT8FrequencySeries *arg1,
	const UINT8FrequencySeries *arg2
)
{
	static const char *func = "XLALAddUINT8FrequencySeries";
	int offset = (arg2->f0 - arg1->f0) / arg1->deltaF;
	REAL8 ratio = XLALUnitRatio(&arg1->sampleUnits, &arg2->sampleUnits);
	unsigned int i;

	/* make sure arguments are compatible */
	if(XLALGPSCmp(&arg1->epoch, &arg2->epoch) || (arg1->deltaF != arg2->deltaF) || XLALIsREAL8FailNaN(ratio))
		XLAL_ERROR_NULL(func, XLAL_EDATA);
	/* FIXME: generalize to relax this requirement */
	if((arg2->f0 < arg1->f0) || (offset + arg2->data->length > arg1->data->length))
		XLAL_ERROR_NULL(func, XLAL_EBADLEN);
	
	/* subtract arg2 from arg1, adjusting the units */
	for(i = 0; i < arg2->data->length; i++) {
		arg1->data->data[offset + i] -= arg2->data->data[i] / ratio;
	}

	return(arg1);
}





UINT8FrequencySeries *XLALMultiplyUINT8FrequencySeries (
	UINT8FrequencySeries *arg1,
	const UINT8FrequencySeries *arg2
)
{
	static const char *func = "XLALMultiplyUINT8FrequencySeries";
	int offset = (arg2->f0 - arg1->f0) / arg1->deltaF;
	REAL8 ratio = XLALUnitRatio(&arg1->sampleUnits, &arg2->sampleUnits);
	unsigned int i;

	/* make sure arguments are compatible */
	if(XLALGPSCmp(&arg1->epoch, &arg2->epoch) || (arg1->deltaF != arg2->deltaF) || XLALIsREAL8FailNaN(ratio))
		XLAL_ERROR_NULL(func, XLAL_EDATA);
	/* FIXME: generalize to relax this requirement */
	if((arg2->f0 < arg1->f0) || (offset + arg2->data->length > arg1->data->length))
		XLAL_ERROR_NULL(func, XLAL_EBADLEN);
	
	/* multiply arg2 by arg1, adjusting the units */
	for(i = 0; i < arg2->data->length; i++) {
		arg1->data->data[offset + i] *= arg2->data->data[i] / ratio;
	}

	return(arg1);
}


